import type {DirectiveBinding, ObjectDirective} from 'vue'
import {alignWithElement} from '../utils'

interface Elm extends Element {
  v$tooltip?: HTMLDivElement
}

const getTooltip = (el: Elm): HTMLDivElement => {
  if (el.v$tooltip != null) {
    return el.v$tooltip
  }

  const tooltip = document.createElement('div')
  tooltip.style.display = "none"
  tooltip.classList.add('tooltip')

  el.addEventListener("mouseenter", () => {
    if (tooltip.parentNode) {
      tooltip.style.display = "block"
      alignWithElement(el, tooltip, "top-center", [0, -6])
    }
  })

  el.addEventListener("mouseleave", () => {
    if (tooltip.parentNode) {
      tooltip.style.display = "none"
    }
  })

  el.v$tooltip = tooltip

  return tooltip
}

const stringify = (value: any) => {
  if (value == null) {
    return value
  } else if (typeof value === 'string') {
    return value.trim()
  } else {
    return value.toString().trim()
  }
}

function destroy(elm: Elm) {
  if (elm.v$tooltip) {
    elm.v$tooltip.remove()
    delete elm.v$tooltip
  }
}

function bind(el: Elm, {value}: DirectiveBinding) {
  const text = stringify(value)
  if (text) {
    const tooltip = getTooltip(el)
    if (!tooltip.parentNode) {
      el.closest('[data-ui-barrier]')?.appendChild(tooltip)
    }
    tooltip.innerHTML = text
  } else {
    destroy(el)
  }
}

export const vTooltip: ObjectDirective<Elm> = {
  mounted: bind,
  updated: bind,
  beforeUnmount: destroy,
}