import {computed, reactive} from 'vue'

export interface Store {
  showSidebar: boolean;
  showTasksPanel: boolean;
  isDragging: boolean;
  mime: FileMime
  lockMime: boolean
  mode: FileMode
  multiple: boolean
  tasks: Array<Task>;
  rootDirectory: CloudDirectory;
  directories: Map<number, CloudDirectory>;
  activeDirectoryId: number;
  // files: Map<number, CloudFile[]>;
  files: Record<number, CloudFile[]>;
  selected: CloudFile[];
}

export const store = reactive<Store>({
  showSidebar: true,
  showTasksPanel: false,
  isDragging: false,
  mime: "all",
  lockMime: false,
  tasks: [],
  mode: 'manage',
  multiple: true,
  directories: new Map(),
  rootDirectory: rootDirectoryEntry(),
  activeDirectoryId: 0,
  files: {},
  selected: [],
});

/** 当前文件视图的关联目录 */
export const activeDirectory = computed(() => {
  if (store.activeDirectoryId === 0) {
    return store.rootDirectory;
  }
  return store.directories.get(store.activeDirectoryId);
});

export function rootDirectoryEntry(): CloudDirectory {
  return {
    id: 0,
    pid: -1,
    title: "根目录",
    createdAt: "",
    sort: Number.MAX_SAFE_INTEGER,
  };
}
