import domAlign, {alignPoint} from "dom-align";

export type Placement =
  | 'top-left'
  | 'top-center'
  | 'top-right'
  | 'bottom-left'
  | 'bottom-center'
  | 'bottom-right'
  | 'left-top'
  | 'left-center'
  | 'left-bottom'
  | 'right-top'
  | 'right-center'
  | 'right-bottom'

// position-align
//
//
//              | top-left       top-center      top-right |
// -------------+------------------------------------------+--------------
//     left-top |                                          | right-top
//              |                                          |
//  left-center |                                          | right-center
//              |                                          |
//  left-bottom |                                          | right-bottom
// -------------+------------------------------------------+--------------
//              | bottom-left  bottom-center  bottom-right |
//
export const positions: Record<Placement, Placement> = {
  // trigger  => popup
  'top-left': 'bottom-left',
  'top-center': 'bottom-center',
  'top-right': 'bottom-right',
  'bottom-left': 'top-left',
  'bottom-center': 'top-center',
  'bottom-right': 'top-right',
  'left-top': 'right-top',
  'left-center': 'right-center',
  'left-bottom': 'right-bottom',
  'right-top': 'left-top',
  'right-center': 'left-center',
  'right-bottom': 'left-bottom',
}

export function alignWithElement(
  trigger: Element,
  popup: HTMLElement,
  placement: Placement,
  offset: [number, number]
) {
  let triggerPoints: string[] = placement.split('-')
  let popupPoints: string[] = positions[placement].split('-')
  switch (triggerPoints[0]) {
    case 'left':
    case 'right':
      triggerPoints = triggerPoints.reverse()
      popupPoints = popupPoints.reverse()
      break
  }
  domAlign(popup, trigger, {
    points: [
      popupPoints.map(c => c[0]).join(''),
      triggerPoints.map(c => c[0]).join(''),
    ],
    offset: offset,
    overflow: {
      adjustX: true,
      adjustY: true,
    },
    useCssTransform: true,
    ignoreShake: true,
  })
}

export function alignWithPoint(
  popup: HTMLElement,
  clientX: number,
  clientY: number,
  placement: Placement,
  offset: [number, number]
) {
  let triggerPoints: string[] = placement.split('-')
  let popupPoints: string[] = positions[placement].split('-')
  switch (triggerPoints[0]) {
    case 'left':
    case 'right':
      triggerPoints = triggerPoints.reverse()
      popupPoints = popupPoints.reverse()
      break
  }
  alignPoint(popup, {clientX, clientY}, {
    points: [
      popupPoints.map(c => c[0]).join(''),
      triggerPoints.map(c => c[0]).join(''),
    ],
    offset: offset,
    overflow: {
      adjustX: true,
      adjustY: true,
    },
    useCssTransform: true,
    ignoreShake: true,
  })
}