import BackendWorker from "../backend?sharedworker&inline";
import {createReplier, emit, isRecvData, isSendData, uniqueId} from "../shared";

type Resolver = (value: any) => void;
type Rejecter = (value: any) => void;

const resolvers: Map<number, [Resolver, Rejecter]> = new Map();

const worker = new BackendWorker();

// 接收来自后端的消息
worker.port.onmessage = (evt) => {
  if (isSendData(evt.data)) {
    const reply = createReplier(worker.port, evt.data);
    const {scope, action, data} = evt.data;
    // console.log(scope, action, data)
    emit(`${scope}:${action}`, data, reply);
  } else if (!isRecvData(evt.data)) {
    console.warn("[wfs] unknown message event:", evt);
  } else if (resolvers.has(evt.data.id)) {
    const {id, error, data} = evt.data;
    const [resolve, reject] = resolvers.get(id)!;
    if (error != null) {
      reject(error);
    } else {
      resolve(data);
    }
  }
};

// 监听页面卸载，通知后端删除关联
window.addEventListener("beforeunload", () => {
  void dispatch("app", "close");
});

window.addEventListener("online", () => {
  return dispatch("net", "status", "on");
});

window.addEventListener("offline", () => {
  return dispatch("net", "status", "off");
});

export function dispatch(scope: "cfg", action: "init", config: ApiConfig): Promise<void>;

export function dispatch(scope: "app", action: "close"): Promise<void>;

export function dispatch(scope: "net", action: "status", data: "on" | "off"): Promise<void>;

export function dispatch(scope: "dir", action: "all"): Promise<Array<CloudDirectory>>;
export function dispatch(scope: "dir", action: "create", data: NewDirEventData): Promise<CloudDirectory>;
export function dispatch(scope: "dir", action: "delete", dir: number): Promise<boolean>;
export function dispatch(scope: "dir", action: "rename", data: RenameDirEventData): Promise<CloudDirectory>;
export function dispatch(scope: "file", action: "rename", data: RenameFileEventData): Promise<CloudFile>;
export function dispatch(scope: "file", action: "delete", files: number[] | number): Promise<boolean>;
export function dispatch(scope: "file", action: "list", data: FilesEventData): Promise<FilesEventResult>

export function dispatch(scope: "task", action: "cleanup"): Promise<void>;
export function dispatch(scope: "task", action: "cancel", data: TaskEventData): Promise<void>;
export function dispatch(scope: "task", action: "create", data: TaskCreateData): Promise<void>;
export function dispatch(scope: "task", action: "remove", data: TaskEventData): Promise<void>;
export function dispatch(scope: "task", action: "resume", data: TaskEventData): Promise<void>;

export function dispatch<T = any>(scope: string, action: string, data?: any): Promise<T> {
  const id = uniqueId();
  return new Promise<T>((resolve, reject) => {
    resolvers.set(id, [resolve, reject]);
    worker.port.postMessage({id, scope, action, data} as SendData);
  });
}
