import {toast} from '../frontend/widgets/UiToastController'

let nextId = 0;

export function uniqueId(): number {
  while (true) {
    const id = ++nextId;
    // https://developer.mozilla.org/zh-CN/docs/Web/JavaScript/Reference/Global_Objects/Number/isSafeInteger
    if (Number.isSafeInteger(id)) {
      return id;
    }
    nextId = 0;
  }
}

export interface AbortionContext {
  onabort(event?: Event): any;
}

export function abortion(ctx: AbortionContext): AbortController {
  const ctrl = new AbortController();
  ctrl.signal.addEventListener("abort", ctx.onabort.bind(ctx));
  return ctrl;
}

export function coerce(val: any): any {
  if (val == null) return val;
  return val instanceof Error ? val.message : val;
}

export function call(fn?: Function, ...args: any[]): void {
  fn && fn(...args);
}

export function humanSize(bytes: number): string {
  if (bytes === 0) return "0B";
  const k = 1024;
  const sizes = ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"];
  const i = Math.floor(Math.log(bytes) / Math.log(k));
  return `${(bytes / Math.pow(k, i)).toPrecision(3)}${sizes[i]}`;
}

export function remove<T>(arr: T[], elm: T): void {
  const index = arr.indexOf(elm);
  index > -1 && arr.splice(index, 1);
}

export function promisify<T>() {
  let resolve: (v: T) => void;
  let reject: (reason?: any) => void;
  const promise = new Promise<T>((res, rej) => {
    resolve = res;
    reject = rej;
  });
  // @ts-ignore
  return {promise, resolve, reject};
}

export function sleep(milliseconds: number): Promise<void> {
  return new Promise(resolve => {
    setTimeout(resolve, milliseconds)
  })
}

export function copyText(text: string, msg = "复制成功"): Promise<void> {
  return navigator.clipboard
    .writeText(text)
    .then(() => toast("success", msg))
    .catch((error) => toast("error", error));
}
